"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (fn, ...args) {
    return (...callArgs) => fn(...args, ...callArgs);
};

module.exports = exports.default;